@@ -28,14 +28,6 @@ namespace Lum {
 
   class LUMAPI DateView : public Control
   {
-  public:
-    class Prefs : public Control::Prefs
-    {
-    public:
-      Prefs(const std::wstring& name);
-      void Initialize();
-    };
-
   private:
     Model::CalendarRef model;
 
