@@ -39,7 +39,7 @@ namespace Lum {
         FillRef   fill[Display::fillCount];
         FrameRef  frame[Display::frameCount];
         ImageRef  image[Display::imageCount];
-        ImageRef  stockImage[stockImageCount];
+        ImageRef  stockImage[imageCount];
 
         FrameRef  emptyFrame;
 
@@ -66,6 +66,8 @@ namespace Lum {
         bool SubdialogsAlwaysFlexVert() const;
 
         DialogActionPosition GetDialogActionPosition() const;
+        bool ImplicitCancelAction() const;
+        bool ImplicitDefaultAction() const;
       };
 
       class DefaultTheme : public Theme
