@@ -40,22 +40,27 @@ namespace Lum {
     class LUMAPI Theme
     {
     public:
-      static const size_t stockImageCount = 8; //! Number of predefined stock images
-
       /**
         List of (optional!) images for enriching visual controls.
        */
       enum StockImage {
-        positiveStockImage  =  0, //! Image for marking a positive action (e.g. for a button)
-        negativeStockImage  =  1, //! Image for marking a negative action (e.g. for a button)
-        defaultStockImage   =  2, //! Image for marking a default action (e.g. for a button)
-        helpStockImage      =  3, //! Image for marking a help action (e.g. for a button)
-        closeStockImage     =  4, //! Image for closing widget
-        addStockImage       =  5, //! Image for adding somethng
-        removeStockImage    =  6, //! Image for removing something
-        editStockImage      =  7  //! Image for editing something
+        imagePositive    =  0, //! Image for marking a positive action (e.g. for a button)
+        imageNegative    =  1, //! Image for marking a negative action (e.g. for a button)
+        imageDefault     =  2, //! Image for marking a default action (e.g. for a button)
+        imageHelp        =  3, //! Image for marking a help action (e.g. for a button)
+        imageClose       =  4, //! Image for closing widget
+        imageAdd         =  5, //! Image for adding somethng
+        imageRemove      =  6, //! Image for removing something
+        imageEdit        =  7, //! Image for editing something
+        imageSave        =  8, //! Image for saving something
+        imageMediaRecord =  9, //! Image for start recordin media playing
+        imageMediaPlay   = 10, //! Image for playing media playing
+        imageMediaPause  = 11, //! Image for pausing media playing
+        imageMediaStop   = 12  //! Image for stoping media playing
       };
 
+      static const size_t imageCount = 13; //! Number of predefined stock images
+
       enum ScrollKnobMode {
         scrollKnobModeSingle,
         scrollKnobModeDouble
@@ -151,6 +156,8 @@ namespace Lum {
       virtual bool SubdialogsAlwaysFlexVert() const = 0;
 
       virtual DialogActionPosition GetDialogActionPosition() const =0;
+      virtual bool ImplicitCancelAction() const =0;
+      virtual bool ImplicitDefaultAction() const =0;
       //@}
     };
   }
