@@ -207,6 +207,8 @@ namespace Lum {
 
         bool SubdialogsAlwaysFlexHoriz() const;
         DialogActionPosition GetDialogActionPosition() const;
+        bool ImplicitCancelAction() const;
+        bool ImplicitDefaultAction() const;
       };
 
     }
