@@ -11,14 +11,6 @@ namespace Lum {
 
   class LUMAPI Scale : public Object
   {
-  public:
-    class Prefs : public Object::Prefs
-    {
-    public:
-      Prefs();
-      void Initialize();
-    };
-
   protected:
     int            from;
     int            to;
