@@ -41,7 +41,7 @@ namespace Lum {
   {
   protected:
     Model::BooleanRef boolModel;
-    Model::IntRef     intModel;
+    Model::SizeTRef   sizeTModel;
     int               index,orgState;
     bool              active;
 
