@@ -40,11 +40,17 @@ namespace Lum {
     Toggle();
     ~Toggle();
 
+    void SetLabel(Object *label);
+    void SetLabel(const std::wstring& label);
+
     void CalcSize();
     void Layout();
 
-    void SetLabel(Object *label);
-    void SetLabel(const std::wstring& label);
+    void PrepareForBackground(OS::DrawInfo* draw);
+
+    static Toggle* Create(const std::wstring& label,
+                          Base::Model* model=NULL,
+                          bool horizontalFlex=false, bool verticalFlex=false);
   };
 }
 
