@@ -21,6 +21,8 @@
 
 #include <Lum/Base/String.h>
 
+#include <Lum/OS/EventLoopGMainLoop.h>
+
 #include <Lum/OS/X11/Display.h>
 
 #include <Lum/Private/Config.h>
@@ -29,92 +31,92 @@
 namespace Lum {
   namespace Audio {
 
+    gboolean GstCallback(GstBus* bus, GstMessage* message, gpointer data)
+    {
+      PlaySoundGst* playSound=(PlaySoundGst*)data;
+
+      if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_STATE_CHANGED) {
+        GstState oldState,newState,pendingState;
+
+        gst_message_parse_state_changed(message,&oldState,&newState,&pendingState);
+
+        //std::cout << "State change from " << gst_element_state_get_name(oldState) << " to " << gst_element_state_get_name(newState) << " pending " << gst_element_state_get_name(pendingState) << std::endl;
+        playSound->state=newState;
+      }
+      else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_TAG) {
+        GstTagList *tagList;
+        char       *value;
+
+        gst_message_parse_tag(message,&tagList);
+
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_TITLE,&value);
+        if (value!=NULL) {
+          //std::cout << "Title: " << value << std::endl;
+          g_free(value);
+        }
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_ARTIST,&value);
+        if (value!=NULL) {
+          //std::cout << "Artist: " << value << std::endl;
+          g_free(value);
+        }
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_ALBUM,&value);
+        if (value!=NULL) {
+          //std::cout << "Album: " << value << std::endl;
+          g_free(value);
+        }
+        /*gst_tag_list_get_string(tagList,GST_TAG_DATE,&value);
+        std::cout << "Date: " << value << std::endl;
+        g_free(value);*/
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_GENRE,&value);
+        if (value!=NULL) {
+          //std::cout << "Genre: " << value << std::endl;
+          g_free(value);
+        }
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_COMMENT,&value);
+        if (value!=NULL) {
+          //std::cout << "Comment: " << value << std::endl;
+          g_free(value);
+        }
+        value=NULL;
+        gst_tag_list_get_string(tagList,GST_TAG_LOCATION,&value);
+        if (value!=NULL) {
+          //std::cout << "Location: " << value << std::endl;
+          g_free(value);
+        }
+
+        gst_tag_list_free(tagList);
+      }
+      else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_EOS) {
+        //std::cout << "Message: EOS" << std::endl;
+      }
+      else {
+        //std::cout << "Message: " << gst_message_type_get_name(GST_MESSAGE_TYPE(message)) << std::endl;
+      }
+
+      return false;
+    }
+
     PlaySoundGst::PlaySoundGst()
-    : gstMsg(new ::Lum::Model::Action()),
-      bin(NULL)
+    : bin(NULL),
+      gsource(NULL)
     {
-      AttachModel(gstMsg);
     }
 
     PlaySoundGst::~PlaySoundGst()
     {
       if (bin!=NULL) {
-        dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->RemoveGstListener(gstMsg.Get());
-
         gst_element_set_state(bin,GST_STATE_NULL);
         ::gst_object_unref(GST_OBJECT(bin));
       }
 
-      UnattachModel(gstMsg);
-    }
-
-    void PlaySoundGst::Resync(::Lum::Base::Model* model, const ::Lum::Base::ResyncMsg& msg)
-    {
-      if (model==gstMsg && gstMsg->IsFinished()) {
-        GstMessage *message=static_cast<const Lum::OS::X11::Display::GstMsg*>(&msg)->message;
-
-        if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_STATE_CHANGED) {
-          GstState oldState,newState,pendingState;
-
-          gst_message_parse_state_changed(message,&oldState,&newState,&pendingState);
-
-          //std::cout << "State change from " << gst_element_state_get_name(oldState) << " to " << gst_element_state_get_name(newState) << " pending " << gst_element_state_get_name(pendingState) << std::endl;
-          state=newState;
-        }
-        else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_TAG) {
-          GstTagList *tagList;
-          char       *value;
-
-          gst_message_parse_tag(message,&tagList);
-
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_TITLE,&value);
-          if (value!=NULL) {
-            //std::cout << "Title: " << value << std::endl;
-            g_free(value);
-          }
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_ARTIST,&value);
-          if (value!=NULL) {
-            //std::cout << "Artist: " << value << std::endl;
-            g_free(value);
-          }
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_ALBUM,&value);
-          if (value!=NULL) {
-            //std::cout << "Album: " << value << std::endl;
-            g_free(value);
-          }
-          /*gst_tag_list_get_string(tagList,GST_TAG_DATE,&value);
-          std::cout << "Date: " << value << std::endl;
-          g_free(value);*/
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_GENRE,&value);
-          if (value!=NULL) {
-            //std::cout << "Genre: " << value << std::endl;
-            g_free(value);
-          }
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_COMMENT,&value);
-          if (value!=NULL) {
-            //std::cout << "Comment: " << value << std::endl;
-            g_free(value);
-          }
-          value=NULL;
-          gst_tag_list_get_string(tagList,GST_TAG_LOCATION,&value);
-          if (value!=NULL) {
-            //std::cout << "Location: " << value << std::endl;
-            g_free(value);
-          }
-
-          gst_tag_list_free(tagList);
-        }
-        else if (GST_MESSAGE_TYPE(message)==GST_MESSAGE_EOS) {
-          //std::cout << "Message: EOS" << std::endl;
-        }
-        else {
-          //std::cout << "Message: " << gst_message_type_get_name(GST_MESSAGE_TYPE(message)) << std::endl;
-        }
+      if (gsource!=NULL) {
+        g_source_destroy(gsource);
+        g_source_unref(gsource);
       }
     }
 
@@ -162,23 +164,30 @@ namespace Lum {
           ::g_object_set(G_OBJECT(source),"location",Lum::Base::WStringToString(filename).c_str(),NULL);
 
           bin=pipeline;
-
-          dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->AddGstListener(bin,gstMsg.Get());
         }
         else {
           bin=::gst_element_factory_make("playbin","playbin");
           if (bin!=NULL) {
             gchar *uri=::g_filename_to_uri(Lum::Base::WStringToString(filename).c_str(),NULL,NULL);
 
+            assert(uri!=NULL);
+
             ::g_object_set(G_OBJECT(bin),"uri",uri,NULL);
-            dynamic_cast<Lum::OS::X11::Display*>(Lum::OS::display)->AddGstListener(bin,gstMsg.Get());
           }
         }
+
+        if (bin!=NULL) {
+          OS::GMainLoopEventLoop* eventLoop=dynamic_cast<OS::GMainLoopEventLoop*>(OS::display->GetEventLoop());
+
+          gsource=::gst_bus_create_watch(gst_pipeline_get_bus(GST_PIPELINE(bin)));
+          ::g_source_set_callback(gsource,(GSourceFunc)GstCallback,this,NULL);
+          g_source_attach(gsource,eventLoop->context);
+        }
       }
 
       if (bin!=NULL) {
         ::gst_element_set_state(bin,GST_STATE_PAUSED);
-       ::gst_element_seek(bin,
+        ::gst_element_seek(bin,
                            1.0,
                            GST_FORMAT_TIME,
                            GST_SEEK_FLAG_FLUSH,
@@ -200,6 +209,6 @@ namespace Lum {
       if (bin!=NULL) {
         ::gst_element_set_state(bin,GST_STATE_NULL);
       }
-   }
+    }
   }
 }
