@@ -1410,15 +1410,15 @@ namespace Lum {
         return -(operator-(b,a));
       }
 
-      // It is now same to assume that a>b...
+      // It is now save to assume that a>b...
 
       long result=0;
 
       Calendar cb(b);
 
       while (cb.year<a.year) {
-        result+=cb.GetDaysOfCurrentYear()-cb.GetDayOfYear();
-        cb.AddDays(cb.GetDaysOfCurrentYear()-cb.GetDayOfYear());
+        result+=cb.GetDaysOfCurrentYear()-cb.GetDayOfYear()+1;
+        cb.AddDays(cb.GetDaysOfCurrentYear()-cb.GetDayOfYear()+1);
       }
 
       result+=a.GetDayOfYear()-cb.GetDayOfYear();
