@@ -39,6 +39,10 @@ namespace Lum {
       const wchar_t *replacement;
     };
 
+    //
+    // class Catalog
+    //
+
     class Catalog
     {
     public:
@@ -53,17 +57,16 @@ namespace Lum {
     private:
       void AddText(const std::wstring& key, const std::wstring& text);
     public:
-      std::wstring GetText(const std::wstring& key, const std::wstring& defaultValue) const;
+      std::wstring GetText(const std::wstring& key, const std::wstring& defaultValue, bool warn=true) const;
 
       static Catalog* LoadCatalog(const std::wstring& locale, const std::wstring& path, Type type);
     };
 
-    std::wstring L10N::locale;
-
     static std::auto_ptr<Catalog> libCatalog;
     static bool                   libCatalogSearched=false;
     static std::auto_ptr<Catalog> appCatalog;
     static bool                   appCatalogSearched=false;
+    static bool                   appCatalogFallback=false;
 
 #if defined(__WIN32__) || defined(WIN32)
     static StringMapping languageMapping[]={
@@ -80,7 +83,7 @@ namespace Lum {
       entries.insert(std::make_pair(key,text));
     }
 
-    std::wstring Catalog::GetText(const std::wstring& key, const std::wstring& defaultValue) const
+    std::wstring Catalog::GetText(const std::wstring& key, const std::wstring& defaultValue, bool warn) const
     {
       std::map<std::wstring,std::wstring>::const_iterator iter;
 
@@ -89,11 +92,34 @@ namespace Lum {
         return iter->second;
       }
       else {
-        std::cerr << "Warning: No entry for key '" << Base::WStringToString(key) << "' for the current locale catalog!" << std::endl;
+        if (warn) {
+          std::cerr << "Warning: No entry for key '" << Base::WStringToString(key) << "' in locale catalog!" << std::endl;
+        }
         return defaultValue;
       }
     }
 
+    //
+    // class L10N
+    //
+
+    std::wstring L10N::locale;
+    std::wstring L10N::fallbackLocale;
+
+    std::wstring StripSubData(const std::wstring& data)
+    {
+      size_t dividerPos;
+
+      dividerPos=data.find(L'_');
+
+      if (dividerPos!=std::wstring::npos) {
+        return data.substr(0,dividerPos);
+      }
+      else {
+        return data;
+      }
+    }
+
     void L10N::SetLocale(const std::wstring& locale)
     {
       Lum::Base::L10N::locale=locale;
@@ -116,15 +142,44 @@ namespace Lum {
 #endif
     }
 
+    void L10N::SetFallbackLocale(const std::wstring& locale)
+    {
+      Lum::Base::L10N::fallbackLocale=locale;
+
+#if defined(__WIN32__) || defined(WIN32)
+      size_t dividerPos;
+
+      dividerPos=Lum::Base::L10N::fallbackLocale.find(L'_');
+
+      if (dividerPos!=std::wstring::npos) {
+        std::wstring language=Lum::Base::L10N::fallbackLocale.substr(0,dividerPos);
+
+        for (size_t i=0; languageMapping[i].source!=NULL; i++) {
+          if (language==languageMapping[i].source) {
+            Lum::Base::L10N::fallbackLocale.replace(0,dividerPos,languageMapping[i].replacement);
+            break;
+          }
+        }
+      }
+#endif
+    }
+
     std::wstring L10N::GetLocalizedText(const std::wstring& key, const std::wstring& defaultValue)
     {
       if (!appCatalogSearched) {
         appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(locale,Path::GetApplicationDataDir(),Catalog::typeApplication));
+        if (appCatalog.get()!=NULL) {
+          appCatalogFallback=(StripSubData(locale)==StripSubData(fallbackLocale));
+        }
+        else {
+          appCatalogFallback=true;
+          appCatalog=std::auto_ptr<Catalog>(Catalog::LoadCatalog(fallbackLocale,Path::GetApplicationDataDir(),Catalog::typeApplication));
+        }
         appCatalogSearched=true;
       }
 
       if (appCatalog.get()!=NULL) {
-        return appCatalog->GetText(key,defaultValue);
+        return appCatalog->GetText(key,defaultValue,!appCatalogFallback);
       }
       else {
         return defaultValue;
@@ -220,8 +275,7 @@ namespace Lum {
         catalogPath.AppendDir(L"catalogs");
         paths.push_back(catalogPath);
       }
-
-      if (type==typeLibrary) {
+      else if (type==typeLibrary) {
         tmp=Path::GetDirFromEnvironment("ILLUMINATION_ROOT");
         if (!tmp.empty()) {
           catalogPath.SetNativeDir(tmp);
@@ -241,15 +295,15 @@ namespace Lum {
           catalogPath.AppendDir(L"catalogs");
           paths.push_back(catalogPath);
         }
-
+        /*
         catalogPath.SetNativeDir(Path::GetCWD());
         catalogPath.AppendDir(L"catalogs");
-        paths.push_back(catalogPath);
+        paths.push_back(catalogPath);*/
       }
 
       localeDir=locale;
 
-      if (type==typeLibrary && OS::display!=NULL) {
+      if (type==typeLibrary) {
         localeNames.push_back(std::wstring(L"Illumination")+L"_"+localeDir);
       }
       else if (type==typeApplication && OS::display!=NULL) {
