@@ -319,15 +319,26 @@ namespace Lum {
       }
     }
 
+    void Path::SetRoot(const std::wstring& root)
+    {
+      this->root=root;
+    }
+
     /**
-      Append an aditional directory part to the existing path.
-      @return false, if the path points to a file.
+      Set the initial directory part.
     */
-    bool Path::AppendDir(const std::wstring& dir)
+    void Path::SetDir(const std::wstring& dir)
+    {
+      parts.clear();
+      parts.push_back(dir);
+    }
+
+    /**
+      Append an additional directory part to the existing path.
+    */
+    void Path::AppendDir(const std::wstring& dir)
     {
       parts.push_back(dir);
-
-      return true;
     }
 
     /**
@@ -753,6 +764,44 @@ namespace Lum {
 #endif
     }
 
+    Status Path::RemoveDir(const std::wstring& nativePath)
+    {
+      Status status;
+
+#if defined(__WIN32__) || defined(WIN32)
+      assert(false);
+#else
+      std::string path;
+
+      path=WStringToString(nativePath);
+
+      if (remove(path.c_str())!=0) {
+        status.SetToCurrentErrno();
+      }
+#endif
+
+      return status;
+    }
+
+    Status Path::RemoveFile(const std::wstring& nativePath)
+    {
+      Status status;
+
+#if defined(__WIN32__) || defined(WIN32)
+      assert(false);
+#else
+      std::string path;
+
+      path=WStringToString(nativePath);
+
+      if (remove(path.c_str())!=0) {
+        status.SetToCurrentErrno();
+      }
+#endif
+
+      return status;
+    }
+
     bool operator!=(const Path& a, const Path& b)
     {
       // TODO: Under Windows we have to compare case-insensitive
