@@ -617,10 +617,24 @@ namespace Lum {
     return button;
   }
 
+  Button* Button::Create(const std::wstring& text,
+                         Lum::OS::Theme::StockImage image,
+                         Base::Model* action,
+                         Type type,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    return Create(text,
+                  OS::display->GetTheme()->GetStockImage(image),
+                  action,
+                  type,
+                  horizontalFlex,
+                  verticalFlex);
+  }
+
   Button* Button::CreateOk(Model::Action* action, bool horizontalFlex, bool verticalFlex)
   {
     return Create(_ld(dlgButtonOk),
-                  OS::display->GetTheme()->GetStockImage(OS::Theme::positiveStockImage),
+                  OS::Theme::imagePositive,
                   action,
                   Button::typeCommit,
                   horizontalFlex,verticalFlex);
@@ -629,7 +643,7 @@ namespace Lum {
   Button* Button::CreateCancel(Model::Action* action, bool horizontalFlex, bool verticalFlex)
   {
     return Create(_ld(dlgButtonCancel),
-                  OS::display->GetTheme()->GetStockImage(OS::Theme::negativeStockImage),
+                  OS::Theme::imageNegative,
                   action,
                   Button::typeCancel,
                   horizontalFlex,verticalFlex);
@@ -638,7 +652,7 @@ namespace Lum {
   Button* Button::CreateClose(Model::Action* action, bool horizontalFlex, bool verticalFlex)
   {
     return Create(_ld(dlgButtonClose),
-                  OS::display->GetTheme()->GetStockImage(OS::Theme::defaultStockImage),
+                  OS::Theme::imageDefault,
                   action,
                   Button::typeDefault,
                   horizontalFlex,verticalFlex);
@@ -647,7 +661,7 @@ namespace Lum {
   Button* Button::CreateQuit(Model::Action* action, bool horizontalFlex, bool verticalFlex)
   {
     return Create(_ld(dlgButtonQuit),
-                  OS::display->GetTheme()->GetStockImage(OS::Theme::defaultStockImage),
+                  OS::Theme::imageDefault,
                   action,
                   Button::typeDefault,
                   horizontalFlex,verticalFlex);
@@ -656,8 +670,9 @@ namespace Lum {
   Button* Button::CreateHelp(Model::Action* action, bool horizontalFlex, bool verticalFlex)
   {
     return Create(_ld(dlgButtonHelp),
-                  OS::display->GetTheme()->GetStockImage(OS::Theme::helpStockImage),
+                  OS::Theme::imageHelp,
                   action,
+                  Button::typeNormal,
                   horizontalFlex,verticalFlex);
   }
 }
