@@ -94,5 +94,16 @@ namespace Lum {
 
     PopupControl::Resync(model,msg);
   }
+
+  DateSelect* DateSelect::Create(Base::Model* model, bool horizontalFlex, bool verticalFlex)
+  {
+    DateSelect* object;
+
+    object=new DateSelect();
+    object->SetFlex(horizontalFlex,verticalFlex);
+    object->SetModel(model);
+
+    return object;
+  }
 }
 
