@@ -76,43 +76,43 @@ namespace Lum {
 
     OS::DrawInfo *draw=GetDrawInfo();
 
-    DrawBackground(x,y,w,h);
+    if (!model.Valid() || model->IsNull()) {
+      return;
+    }
 
-    if (model.Valid() && !model->IsNull()) {
-      double percent=(model->Get()-model->GetMin())/(model->GetMax()-model->GetMin());
+    double percent=(model->Get()-model->GetMin())/(model->GetMax()-model->GetMin());
 
-      if (width>=height) {
-        Lum::OS::FontExtent extent;
-        std::wstring        label(Base::NumberToWString(lround(percent*100))+L"%");
-        OS::FillRef         fill=OS::display->GetFill(OS::Display::hprogressFillIndex);
+    if (width>=height) {
+      Lum::OS::FontExtent extent;
+      std::wstring        label(Base::NumberToWString(lround(percent*100))+L"%");
+      OS::FillRef         fill=OS::display->GetFill(OS::Display::hprogressFillIndex);
 
-        fill->Draw(draw,
-                   this->x,this->y,
-                   width,height,
-                   this->x,this->y,
-                   (size_t)(width*percent),height);
+      fill->Draw(draw,
+                 this->x,this->y,
+                 width,height,
+                 this->x,this->y,
+                 (size_t)(width*percent),height);
 
 
-        font->StringExtent(label,extent);
+      font->StringExtent(label,extent);
 
-        draw->PushForeground(OS::Display::textColor);
-        draw->PushFont(font,0);
-        draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
-                         this->y+(height-font->height)/2+font->ascent,
-                         label);
-        draw->PopFont();
-        draw->PopForeground();
+      draw->PushForeground(OS::Display::textColor);
+      draw->PushFont(font,0);
+      draw->DrawString(this->x+(width-extent.width+extent.left+extent.right)/2,
+                       this->y+(height-font->height)/2+font->ascent,
+                       label);
+      draw->PopFont();
+      draw->PopForeground();
 
-      }
-      else {
-        OS::FillRef fill=OS::display->GetFill(OS::Display::vprogressFillIndex);
+    }
+    else {
+      OS::FillRef fill=OS::display->GetFill(OS::Display::vprogressFillIndex);
 
-        fill->Draw(draw,
-                   this->x,this->y,
-                   width,height,
-                   this->x,this->y+height-(size_t)(height*percent),
-                   width,(size_t)(height*percent));
-      }
+      fill->Draw(draw,
+                 this->x,this->y,
+                 width,height,
+                 this->x,this->y+height-(size_t)(height*percent),
+                 width,(size_t)(height*percent));
     }
   }
 
@@ -123,4 +123,25 @@ namespace Lum {
     }
     Control::Resync(model,msg);
   }
+
+  FuelGauge* FuelGauge::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    FuelGauge *f;
+
+    f=new FuelGauge();
+    f->SetFlex(horizontalFlex,verticalFlex);
+
+    return f;
+  }
+
+  FuelGauge* FuelGauge::Create(Base::Model* model, bool horizontalFlex, bool verticalFlex)
+  {
+    FuelGauge *f;
+
+    f=new FuelGauge();
+    f->SetFlex(horizontalFlex,verticalFlex);
+    f->SetModel(model);
+
+    return f;
+  }
 }
