@@ -20,6 +20,7 @@
 #include <Lum/Menu.h>
 
 #include <cctype>
+#include <cstdlib>
 
 #include <Lum/Base/Util.h>
 
@@ -433,7 +434,11 @@ namespace Lum {
   /* ---------- Menustrip stuff ------------------*/
 
   MenuStrip::MenuStrip()
-  : selection(NULL),stickyMode(false)
+  : selection(NULL),
+    x(0),
+    y(0),
+    pullDownMenu(NULL),
+    stickyMode(false)
   {
     SetBackground(OS::display->GetFill(OS::Display::menuStripBackgroundFillIndex));
 
@@ -443,13 +448,13 @@ namespace Lum {
   /**
     Finds the menu entry (@otype{PullDownMenu}) that opens the given (sub)menu).
   */
-  std::list<Object*>::iterator MenuStrip::FindLabel(Menu* menu)
+  std::list<PullDownMenu*>::iterator MenuStrip::FindLabel(Menu* menu)
   {
-    std::list<Object*>::iterator iter;
+    std::list<PullDownMenu*>::iterator iter;
 
     iter=list.begin();
     while (iter!=list.end()) {
-      if (dynamic_cast<PullDownMenu*>(*iter)->subMenu==menu) {
+      if ((*iter)->subMenu==menu) {
         return iter;
       }
 
@@ -480,7 +485,7 @@ namespace Lum {
   */
   void MenuStrip::SelectPrevious()
   {
-    std::list<Object*>::iterator object;
+    std::list<PullDownMenu*>::iterator object;
 
     if (selection==NULL) {
       return;
@@ -489,11 +494,11 @@ namespace Lum {
     object=FindLabel(selection);
 
     if (object==list.begin()) {
-      SetSelection(dynamic_cast<PullDownMenu*>(*list.rbegin())->subMenu);
+      SetSelection((*list.rbegin())->subMenu);
     }
     else {
       object--;
-      SetSelection(dynamic_cast<PullDownMenu*>(*object)->subMenu);
+      SetSelection((*object)->subMenu);
     }
   }
 
@@ -502,7 +507,7 @@ namespace Lum {
   */
   void MenuStrip::SelectNext()
   {
-    std::list<Object*>::iterator object;
+    std::list<PullDownMenu*>::iterator object;
 
     if (selection==NULL) {
       return;
@@ -513,10 +518,10 @@ namespace Lum {
     object++;
 
     if (object!=list.end()) {
-      SetSelection(dynamic_cast<PullDownMenu*>(*object)->subMenu);
+      SetSelection((*object)->subMenu);
     }
     else if (list.begin()!=list.end()) {
-      SetSelection(dynamic_cast<PullDownMenu*>(*list.begin())->subMenu);
+      SetSelection((*list.begin())->subMenu);
     }
   }
 
@@ -526,7 +531,7 @@ namespace Lum {
   */
   Object* MenuStrip::GetMouseSelected() const
   {
-    std::list<Object*>::const_iterator object;
+    std::list<PullDownMenu*>::const_iterator object;
 
     object=list.begin();
     while (object!=list.end()) {
@@ -545,6 +550,7 @@ namespace Lum {
     PullDownMenu *menu;
 
     menu=new PullDownMenu();
+    menu->SetParent(this);
     menu->SetFlex(false,true);
     menu->SetLabel(label);
     menu->SetHotkey(sc);
@@ -556,7 +562,7 @@ namespace Lum {
       subMenu->SetReference(menu);
     }
 
-    Add(menu);
+    list.push_back(menu);
   }
 
   void MenuStrip::AddPullDownMenu(const std::wstring& label, Menu* subMenu)
@@ -584,9 +590,20 @@ namespace Lum {
     AddPullDownMenu(text,shortcut,subMenu);
   }
 
+  bool MenuStrip::VisitChildren(Visitor &visitor, bool onlyVisible)
+  {
+    for (std::list<PullDownMenu*>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
+      if (!visitor.Visit(*iter)) {
+        return false;
+      }
+    }
+
+    return true;
+  }
+
   void MenuStrip::CalcSize()
   {
-    std::list<Object*>::const_iterator object;
+    std::list<PullDownMenu*>::const_iterator object;
 
     width=0;
     height=0;
@@ -600,12 +617,12 @@ namespace Lum {
       ++object;
     }
 
-    List::CalcSize();
+    Group::CalcSize();
   }
 
   void MenuStrip::Layout()
   {
-    std::list<Object*>::const_iterator object;
+    std::list<PullDownMenu*>::const_iterator object;
     int                                  pos;
 
     pos=x;
@@ -618,7 +635,7 @@ namespace Lum {
       ++object;
     }
 
-    List::Layout();
+    Group::Layout();
   }
 
   /**
@@ -628,7 +645,7 @@ namespace Lum {
   void MenuStrip::PreDrawChild(Object* child)
   {
     OS::DrawInfo *draw=GetDrawInfo();
-    std::list<Object*>::iterator label=FindLabel(selection);
+    std::list<PullDownMenu*>::iterator label=FindLabel(selection);
     if (label!=list.end() && child==*label) {
       draw->selected=true;
     }
@@ -644,6 +661,68 @@ namespace Lum {
     draw->selected=false;
   }
 
+  bool MenuStrip::HandleMouseEvent(const OS::MouseEvent& event)
+  {
+    if (!visible) {
+      return false;
+    }
+
+    if (event.type==OS::MouseEvent::down &&
+        PointIsIn(event) &&
+        (event.button==OS::MouseEvent::button1 || event.button==OS::MouseEvent::button3)) {
+
+      stickyMode=false;
+      pullDownMenu=NULL;
+
+      for (std::list<PullDownMenu*>::iterator p=list.begin(); p!=list.end(); ++p) {
+        if ((*p)->PointIsIn(event)) {
+          (*p)->Activate();
+          x=event.x;
+          y=event.y;
+          pullDownMenu=*p;
+          break;
+        }
+      }
+
+      return true;
+    }
+    else if (event.type==OS::MouseEvent::move &&
+             PointIsIn(event) &&
+             (event.qualifier==OS::qualifierButton1 || event.qualifier==OS::qualifierButton3 || stickyMode)) {
+      for (std::list<PullDownMenu*>::iterator p=list.begin(); p!=list.end(); ++p) {
+        if ((*p)->PointIsIn(event)) {
+          (*p)->Activate();
+        }
+      }
+      return true;
+    }
+    else if (event.type==OS::MouseEvent::up) {
+      EvaluateMouseUp(event);
+    }
+
+    return false;
+  }
+
+  void MenuStrip::EvaluateMouseUp(const OS::MouseEvent& event)
+  {
+    if (event.type==OS::MouseEvent::up &&
+        PointIsIn(event) &&
+        (event.button==OS::MouseEvent::button1 || event.button==OS::MouseEvent::button3)) {
+      if ((size_t)std::abs(event.x-x)<=Lum::OS::display->GetTheme()->GetMouseClickHoldSensitivity() &&
+          (size_t)std::abs(event.y-y)<=Lum::OS::display->GetTheme()->GetMouseClickHoldSensitivity()) {
+        stickyMode=true;
+      }
+      else {
+        for (std::list<PullDownMenu*>::iterator p=list.begin(); p!=list.end(); ++p) {
+          if ((*p)->PointIsIn(event) && pullDownMenu!=*p) {
+            stickyMode=true;
+            break;
+          }
+        }
+      }
+    }
+  }
+
   /* ---------- PullDownMenu stuff ------------------*/
 
   PullDownMenu::PullDownMenu()
@@ -712,24 +791,6 @@ namespace Lum {
     Object::Layout();
   }
 
-  bool PullDownMenu::HandleMouseEvent(const OS::MouseEvent& event)
-  {
-    if (!visible) {
-      return false;
-    }
-
-    if (event.type==OS::MouseEvent::down && PointIsIn(event) && (event.button==OS::MouseEvent::button1 || event.button==OS::MouseEvent::button3)) {
-      strip->SetSelection(subMenu);
-      return true;
-    }
-    else if (event.type==OS::MouseEvent::move && subMenu!=NULL && !subMenu->IsOpen() && PointIsIn(event) && (event.qualifier==OS::qualifierButton1 || event.qualifier==OS::qualifierButton3 || strip->stickyMode)) {
-      strip->SetSelection(subMenu);
-      return true;
-    }
-
-    return false;
-  }
-
   void PullDownMenu::PrepareForBackground(OS::DrawInfo* draw)
   {
     draw->activated=IsMouseActive();
@@ -763,6 +824,7 @@ namespace Lum {
 
   void PullDownMenu::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
+    // TODO: Move this code to the strip
     if (model==this->shortcutAction && this->shortcutAction->IsFinished()) {
       strip->stickyMode=true;
       strip->SetSelection(subMenu);
@@ -771,6 +833,13 @@ namespace Lum {
     Object::Resync(model,msg);
   }
 
+  void PullDownMenu::Activate()
+  {
+    if (subMenu!=NULL && !subMenu->IsOpen()) {
+      strip->SetSelection(subMenu);
+    }
+  }
+
   /* ---------- Menu entry stuff ------------------*/
 
   MenuEntry::MenuEntry()
@@ -1795,6 +1864,19 @@ namespace Lum {
     return window->Open(true);
   }
 
+  bool Menu::OpenInStickyMode()
+  {
+    bool result;
+
+    result=window->Open(true);
+
+    if (result) {
+      stickyMode=true;
+    }
+
+    return result;
+  }
+
   /**
     Close menu.
   */
@@ -2008,7 +2090,8 @@ namespace Lum {
     OS::KeyEvent   *key;
 
     if ((mouse=dynamic_cast<OS::MouseEvent*>(event))!=NULL) {
-      if (mouse->type==OS::MouseEvent::up && (mouse->button==OS::MouseEvent::button3 || mouse->button==OS::MouseEvent::button1)) {
+      if (mouse->type==OS::MouseEvent::up &&
+          (mouse->button==OS::MouseEvent::button3 || mouse->button==OS::MouseEvent::button1)) {
         menu=GetMenuWithMouseSelection();
 
         if (menu!=NULL && menu->CursorIsIn()) {
@@ -2022,17 +2105,20 @@ namespace Lum {
         else {
           /* No menu item selected */
           strip=GetStrip();
-          if (strip!=NULL && !strip->stickyMode && strip->MouseIsIn()) {
-            object=strip->GetMouseSelected();
-            if ((object!=NULL) && (dynamic_cast<PullDownMenu*>(object)->subMenu=GetTop())) {
-              strip->stickyMode=true;
+          if (strip!=NULL) {
+            // Transform menu coordinates to strip coordinates (in another window)
+            mouse->x=mouse->x+GetWindow()->GetX()-strip->GetWindow()->GetX();
+            mouse->y=mouse->y+GetWindow()->GetY()-strip->GetWindow()->GetY();
+            strip->EvaluateMouseUp(*mouse);
+
+            if (!strip->stickyMode) {
+              EndAction();
             }
           }
-          else {
+          else if (!stickyMode) {
             EndAction();
           }
         }
-
       }
       else if (mouse->type==OS::MouseEvent::down && (mouse->button==OS::MouseEvent::button3 || mouse->button==OS::MouseEvent::button1)) {
         menu=GetMenuWithMouseSelection();
@@ -2145,6 +2231,7 @@ namespace Lum {
       top->Draw(redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
     }
     else if (model==window->GetPreInitAction() && window->GetPreInitAction()->IsFinished()) {
+      stickyMode=false;
       PreInit();
     }
     else if (model==window->GetEventAction() && window->GetEventAction()->IsFinished()) {
