@@ -26,6 +26,8 @@
 #include <Lum/Base/String.h>
 #include <Lum/Base/Util.h>
 
+#include <Lum/OS/Manager/Repository.h>
+
 namespace Lum {
   namespace OS {
     namespace Base {
@@ -40,8 +42,7 @@ namespace Lum {
         multiClickTime(200),
         appName(L""),
         theme(NULL),
-        eventLoop(NULL),
-        keyboardHandler(NULL)
+        eventLoop(NULL)
       {
         int i=1;
 
@@ -78,6 +79,10 @@ namespace Lum {
 
       void Display::Close()
       {
+        if (GetKeyboardManager()!=NULL) {
+          GetKeyboardManager()->Shutdown();
+        }
+
         for (std::map<size_t,FontRef>::iterator iter=propFonts.begin(); iter!=propFonts.end(); ++iter) {
           iter->second=NULL;
         }
@@ -89,8 +94,6 @@ namespace Lum {
         propFont=NULL;
         fixedFont=NULL;
 
-        delete keyboardHandler;
-
         delete theme;
       }
 
@@ -594,6 +597,11 @@ namespace Lum {
         return standard.Get();
       }
 
+      OS::Font* Display::GetFont(size_t scale) const
+      {
+        return GetFont(fontTypeProportional,scale);
+      }
+
       OS::Font* Display::GetFont(FontType type, size_t scale) const
       {
         switch (type) {
@@ -653,9 +661,9 @@ namespace Lum {
         return NULL;
       }
 
-      KeyboardHandler* Display::GetKeyboardHandler() const
+      Manager::KeyboardManager* Display::GetKeyboardManager() const
       {
-        return keyboardHandler;
+        return Manager::repository->GetKeyboardManager();
       }
 
       OS::Display::ByteOrder Display::GetSystemByteOrder() const
