@@ -443,8 +443,17 @@ namespace Lum {
 
       Theme::DialogActionPosition Theme::GetDialogActionPosition() const
       {
-        //return actionPosHorizButtonRowBelow;
-        return actionPosVertButtonRowRight;
+        return actionPosHorizButtonRowBelow;
+      }
+
+      bool Theme::ImplicitCancelAction() const
+      {
+        return false;
+      }
+
+      bool Theme::ImplicitDefaultAction() const
+      {
+        return false;
       }
 
       /**
