@@ -53,6 +53,7 @@ namespace Lum {
         maxWidth(32000),maxHeight(32000),
         horizontalPos(positionOS),
         verticalPos(positionOS),
+        opacity(1.0),
         modalCount(0),
         parent(NULL),
         draw(NULL),
@@ -258,6 +259,18 @@ namespace Lum {
         verticalPos=vert;
       }
 
+      bool Window::SetOpacity(double opacity)
+      {
+        this->opacity=opacity;
+
+        return false;
+      }
+
+      double Window::GetOpacity() const
+      {
+        return opacity;
+      }
+
       int Window::GetX() const
       {
         return x;
