@@ -19,6 +19,8 @@
 
 #include <Lum/OS/Win32/Win32Theme.h>
 
+#include <iostream>
+
 #include <uxtheme.h>
 #include <tmschema.h>
 #include <Lum/OS/Win32/OSAPI.h>
@@ -357,6 +359,18 @@ namespace Lum {
             height=GetSystemMetrics(SM_CYMENUCHECK)-4;
           }
           break;
+        case Display::radioImageIndex:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
+        case Display::checkImageIndex:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
+        case Display::ledImageIndex:
+          width=GetSystemMetrics(SM_CXMENUCHECK);
+          height=GetSystemMetrics(SM_CYMENUCHECK);
+          break;
         }
       }
 
@@ -838,6 +852,89 @@ namespace Lum {
             draw->PopPen();
           }
           break;
+        case Display::radioImageIndex:
+          if (HasTheming()) {
+            if (draw->selected) {
+              if (draw->activated) {
+                style=RBS_CHECKEDHOT;
+              }
+              else if (draw->disabled) {
+                style=RBS_CHECKEDDISABLED;
+              }
+              else {
+                style=RBS_CHECKEDNORMAL;
+              }
+            }
+            else if (draw->activated) {
+              style=RBS_UNCHECKEDHOT;
+            }
+            else if (draw->disabled) {
+              style=RBS_UNCHECKEDDISABLED;
+            }
+            else {
+              style=RBS_UNCHECKEDNORMAL;
+            }
+
+            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_RADIOBUTTON,style,&rect,NULL);
+          }
+          else {
+            /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONRADIO | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
+        case Display::checkImageIndex:
+          if (HasTheming()) {
+            if (draw->selected) {
+              if (draw->activated) {
+                style=CBS_CHECKEDHOT;
+              }
+              else if (draw->disabled) {
+                style=CBS_CHECKEDDISABLED;
+              }
+              else {
+                style=CBS_CHECKEDNORMAL;
+              }
+            }
+            else if (draw->activated) {
+              style=CBS_UNCHECKEDHOT;
+            }
+              else if (draw->disabled) {
+                style=CBS_UNCHECKEDDISABLED;
+              }
+            else {
+              style=CBS_UNCHECKEDNORMAL;
+            }
+
+            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_CHECKBOX,style,&rect,NULL);
+          }
+          else {
+           /* ignore */ ::DrawFrameControl(dc,&rect,
+                                            DFC_BUTTON,
+                                            DFCS_BUTTONCHECK | (draw->selected ? DFCS_CHECKED : 0));
+          }
+          break;
+        case Display::ledImageIndex:
+          if (draw->selected) {
+            draw->PushForeground(GetSysColor(COLOR_HIGHLIGHT));
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+          else {
+            draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
+            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
+            draw->PopForeground();
+          }
+
+          draw->PushForeground(GetSysColor(COLOR_3DHIGHLIGHT));
+          draw->DrawArc(x+width / 4,y+height / 4,
+                       width / 2,height / 2,60*64,90*64);
+          draw->PopForeground();
+
+          draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
+          draw->DrawArc(x,y,width-1,height-1,0*64,360*64);
+          draw->PopForeground();
+          break;
         }
 
         SetTextAlign(dc,TA_BASELINE+TA_LEFT);
@@ -960,6 +1057,12 @@ namespace Lum {
             topBorder=GetSystemMetrics(SM_CYEDGE);
             bottomBorder=topBorder;
           }
+          // We manually add some space in all directions, since there is no way to
+          // get a theme specific value for the space between the actual border and its content.
+          leftBorder+=3;
+          rightBorder+=3;
+          topBorder+=3;
+          bottomBorder+=3;
           break;
         case Display::columnBackgroundFillIndex:
         case Display::columnLeftBackgroundFillIndex:
@@ -986,6 +1089,12 @@ namespace Lum {
             topBorder=GetSystemMetrics(SM_CYEDGE);
             bottomBorder=topBorder;
           }
+          // We manually add some space in all directions, since there is no way to
+          // get a theme specific value for the space between the actual border and its content.
+          leftBorder+=3;
+          rightBorder+=3;
+          topBorder+=3;
+          bottomBorder+=3;
           break;
         case Display::hprogressBackgroundFillIndex:
           if (HasTheming()) {
@@ -1115,9 +1224,9 @@ namespace Lum {
             content.right=0;
             content.top=0;
             content.bottom=0;
-            
+
             MyGetThemeBackgroundExtent(tabTheme,NULL,TABP_TOPTABITEM,TTIS_NORMAL,&content,&extent);
-            
+
             leftBorder=-extent.left;
             rightBorder=extent.right;
             topBorder=-extent.top;
@@ -1511,7 +1620,7 @@ namespace Lum {
             rect.top=yOff;
             rect.right=xOff+width;
             rect.bottom=yOff+height;
-            
+
             /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
             return;
           }
@@ -1536,12 +1645,12 @@ namespace Lum {
           }
           else {
             /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_BTNFACE));
-            
+
             rect.left=xOff;
             rect.top=yOff;
             rect.right=xOff+width;
             rect.bottom=yOff+height;
-            
+
             /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
             return;
           }
@@ -1772,7 +1881,7 @@ namespace Lum {
             draw->DrawLine(xOff+1,yOff+1,xOff+1  ,yOff+height-1);
 
             draw->PopForeground();
-            
+
             return;
           }
           break;
@@ -1956,12 +2065,12 @@ namespace Lum {
           break;
         case Display::graphBackgroundFillIndex:
           /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          
+
           rect.left=xOff;
           rect.top=yOff;
           rect.right=xOff+width;
           rect.bottom=yOff+height;
-            
+
           /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
           return;
         case Display::toolbarBackgroundFillIndex:
@@ -1975,23 +2084,23 @@ namespace Lum {
           else {
 #endif
           /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          
+
           rect.left=xOff;
           rect.top=yOff;
           rect.right=xOff+width;
           rect.bottom=yOff+height;
-          
+
           /* ignore */ DrawEdge(dc,&rect,EDGE_RAISED,BF_RECT);
          //}
           return;
         case Display::popupWindowBackgroundFillIndex:
           /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          
+
           rect.left=xOff;
           rect.top=yOff;
           rect.right=xOff+width;
           rect.bottom=yOff+height;
-          
+
           /* ignore */ DrawEdge(dc,&rect,EDGE_RAISED,BF_RECT);
           return;
         case Display::dialogWindowBackgroundFillIndex:
@@ -2008,22 +2117,25 @@ namespace Lum {
           return;
         case Display::scrolledBackgroundFillIndex:
           /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          
+
           rect.left=xOff;
           rect.top=yOff;
           rect.right=xOff+width;
           rect.bottom=yOff+height;
-          
+
           /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
           return;
+        case Display::hscrollBackgroundFillIndex:
+        case Display::vscrollBackgroundFillIndex:
+          return;
         case Display::boxedBackgroundFillIndex:
           /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          
+
           rect.left=xOff;
           rect.top=yOff;
           rect.right=xOff+width;
           rect.bottom=yOff+height;
-          
+
           /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
           return;
         case Display::plateBackgroundFillIndex:
@@ -2048,6 +2160,7 @@ namespace Lum {
         case Display::statusbarBackgroundFillIndex:
           return;
         default:
+          std::cerr << "Missing background support for background " << fill << std::endl;
           assert(false);
           break;
         }
@@ -2335,6 +2448,13 @@ namespace Lum {
         color[Display::whiteColor]=RGB(0xff,0xff,0xff);
         color[Display::tabTextColor]=GetSysColor(COLOR_BTNTEXT);
         color[Display::tabTextSelectColor]=GetSysColor(COLOR_BTNTEXT);
+        color[Display::editTextColor]=GetSysColor(COLOR_WINDOWTEXT);
+        if (HasTheming()) {
+          color[Display::editTextFillColor]=GetSysColor(COLOR_WINDOWTEXT);
+        }
+        else {
+          color[Display::editTextFillColor]=GetSysColor(COLOR_HIGHLIGHTTEXT);
+        }
 
         fill[Display::backgroundFillIndex]=new Fill(Display::backgroundFillIndex);
         fill[Display::tableBackgroundFillIndex]=new Fill(Display::tableBackgroundFillIndex);
@@ -2412,6 +2532,9 @@ namespace Lum {
         image[Display::menuCheckImageIndex]=new ThemeImage(Display::menuCheckImageIndex);
         image[Display::menuRadioImageIndex]=new ThemeImage(Display::menuRadioImageIndex);
         image[Display::closeImageIndex]=new ThemeImage(Display::closeImageIndex);
+        image[Display::radioImageIndex]=new ThemeImage(Display::radioImageIndex);
+        image[Display::checkImageIndex]=new ThemeImage(Display::checkImageIndex);
+        image[Display::ledImageIndex]=new ThemeImage(Display::ledImageIndex);
       }
 
       Win32Theme::~Win32Theme()
@@ -2467,143 +2590,6 @@ namespace Lum {
         return ((metric.tmAscent+metric.tmDescent)*72)/GetDeviceCaps(dynamic_cast<Display*>(OS::display)->hdc,LOGPIXELSY);
       }
 
-      void Win32Theme::GetObjectSize(ObjectType object,
-                                     size_t& width,
-                                     size_t& height) const
-      {
-        switch (object) {
-        case objCheck:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
-        case objRadio:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
-        case objLED:
-          width=GetSystemMetrics(SM_CXMENUCHECK);
-          height=GetSystemMetrics(SM_CYMENUCHECK);
-          break;
-        }
-      }
-
-      bool Win32Theme::NeedsObjectParentDraw(ObjectType object) const
-      {
-        switch (object) {
-        case objCheck:
-        case objRadio:
-        case objLED:
-          return true;
-        }
-
-        assert(false);
-      }
-
-      void Win32Theme::DrawObject(OS::DrawInfo* draw,
-                                  ObjectType object,
-                                  int x, int y, size_t width, size_t height,
-                                  int cX, int cY, size_t cWidth, size_t cHeight)
-      {
-        HDC  dc=dynamic_cast<DrawInfo*>(draw)->dc;
-        RECT rect;
-        int  style;
-
-        rect.left=x;
-        rect.top=y;
-        rect.right=x+width;
-        rect.bottom=y+height;
-
-        SetTextAlign(dc,TA_TOP+TA_LEFT);
-
-        switch (object) {
-        case objCheck:
-          if (HasTheming()) {
-            if (draw->selected) {
-              if (draw->activated) {
-                style=CBS_CHECKEDHOT;
-              }
-              else if (draw->disabled) {
-                style=CBS_CHECKEDDISABLED;
-              }
-              else {
-                style=CBS_CHECKEDNORMAL;
-              }
-            }
-            else if (draw->activated) {
-              style=CBS_UNCHECKEDHOT;
-            }
-              else if (draw->disabled) {
-                style=CBS_UNCHECKEDDISABLED;
-              }
-            else {
-              style=CBS_UNCHECKEDNORMAL;
-            }
-
-            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_CHECKBOX,style,&rect,NULL);
-          }
-          else {
-           /* ignore */ ::DrawFrameControl(dc,&rect,
-                                            DFC_BUTTON,
-                                            DFCS_BUTTONCHECK | (draw->selected ? DFCS_CHECKED : 0));
-          }
-          break;
-        case objRadio:
-          if (HasTheming()) {
-            if (draw->selected) {
-              if (draw->activated) {
-                style=RBS_CHECKEDHOT;
-              }
-              else if (draw->disabled) {
-                style=RBS_CHECKEDDISABLED;
-              }
-              else {
-                style=RBS_CHECKEDNORMAL;
-              }
-            }
-            else if (draw->activated) {
-              style=RBS_UNCHECKEDHOT;
-            }
-            else if (draw->disabled) {
-              style=RBS_UNCHECKEDDISABLED;
-            }
-            else {
-              style=RBS_UNCHECKEDNORMAL;
-            }
-
-            /* ignore */ MyDrawThemeBackground(buttonTheme,dc,BP_RADIOBUTTON,style,&rect,NULL);
-          }
-          else {
-            /* ignore */ ::DrawFrameControl(dc,&rect,
-                                            DFC_BUTTON,
-                                            DFCS_BUTTONRADIO | (draw->selected ? DFCS_CHECKED : 0));
-          }
-          break;
-        case objLED:
-          if (draw->selected) {
-            draw->PushForeground(GetSysColor(COLOR_HIGHLIGHT));
-            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
-            draw->PopForeground();
-          }
-          else {
-            draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
-            draw->FillArc(x,y,width-1,height-1,0*64,360*64);
-            draw->PopForeground();
-          }
-
-          draw->PushForeground(GetSysColor(COLOR_3DHIGHLIGHT));
-          draw->DrawArc(x+width / 4,y+height / 4,
-                       width / 2,height / 2,60*64,90*64);
-          draw->PopForeground();
-
-          draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
-          draw->DrawArc(x,y,width-1,height-1,0*64,360*64);
-          draw->PopForeground();
-          break;
-        }
-
-        SetTextAlign(dc,TA_BASELINE+TA_LEFT);
-      }
-
       size_t Win32Theme::GetSpaceHorizontal(Display::Space space) const
       {
         switch (space) {
@@ -2677,17 +2663,6 @@ namespace Lum {
         return OS::display->GetSpaceHorizontal(Display::spaceInterObject);
       }
 
-
-      size_t Win32Theme::GetEntryHorizTextOffset() const
-      {
-        return 2;
-      }
-
-      size_t Win32Theme::GetEntryVertTextOffset() const
-      {
-        return 2;
-      }
-
       OS::Theme::ScrollKnobMode Win32Theme::GetScrollKnobMode() const
       {
         return OS::Theme::scrollKnobModeSingle;
@@ -2702,10 +2677,10 @@ namespace Lum {
       {
         return false;
       }
-      
+
       OS::Theme::MenuType Win32Theme::GetMenuType() const
       {
-	      return menuTypeStrip;
+        return menuTypeStrip;
       }
     }
   }
