@@ -95,7 +95,7 @@ namespace Lum {
       }
     }
 
-    button=new Button(image);
+    button=Button::Create(image,dec ? knob->GetDecAction() : knob->GetIncAction());
     button->SetType(Button::typeScroll);
     if (vert) {
       button->SetFlex(true,false);
@@ -105,8 +105,6 @@ namespace Lum {
     }
     button->SetPulse(true);
 
-    button->SetModel(dec ? knob->GetDecAction() : knob->GetIncAction());
-
     return button;
   }
 
