@@ -104,12 +104,6 @@ namespace Lum {
 
     boxSize=width-boxFrame->minWidth-2*(knob->GetWidth()/2);
 
-    if (!model.Valid() || model->IsNull()) {
-      DrawBackground(x+boxFrame->leftBorder,y+offset,
-                     width-boxFrame->minWidth,knob->GetHeight());
-      return;
-    }
-
     if (model->GetMin()==model->GetMax()) {
       kStart=0;
     }
@@ -218,17 +212,7 @@ namespace Lum {
     }
     fSize=knob->GetHeight()+boxFrame->minHeight;
 
-
     if (useScale) {
-      draw->PushClipBegin(x,y,w,h);
-      draw->SubClipRegion(scale->GetOX(),scale->GetOY(),scale->GetOWidth(),scale->GetOHeight());
-      draw->PushClipEnd();
-    }
-
-    DrawBackground(x,y,w,h);
-
-    if (useScale) {
-      draw->PopClip();
       scale->Draw(x,y,w,h);
     }
 
@@ -355,6 +339,42 @@ namespace Lum {
     return false;
   }
 
+  Slider* HSlider::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new HSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+
+    return s;
+  }
+
+  Slider* HSlider::Create(Base::Model* model,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new HSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetModel(model);
+
+    return s;
+  }
+
+  Slider* HSlider::Create(Base::Model* model,
+                          bool useScale,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new HSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetScale(useScale);
+    s->SetModel(model);
+
+    return s;
+  }
+
   VSlider::VSlider()
   {
     knob=OS::display->GetImage(OS::Display::vscaleKnobImageIndex);
@@ -381,12 +401,6 @@ namespace Lum {
 
     boxSize=height-boxFrame->minHeight-knob->GetHeight();
 
-    if (!model.Valid() || model->IsNull()) {
-      DrawBackground(x+offset,y+boxFrame->topBorder,
-                     knob->GetWidth(),height-boxFrame->minHeight);
-      return;
-    }
-
     if (model->GetMax()==model->GetMin()) {
       kStart=0;
     }
@@ -495,14 +509,6 @@ namespace Lum {
     fSize=knob->GetWidth()+boxFrame->minWidth;
 
     if (useScale) {
-      draw->PushClipBegin(x,y,w,h);
-      draw->SubClipRegion(scale->GetOX(),scale->GetOY(),scale->GetOWidth(),scale->GetOHeight());
-    }
-
-    DrawBackground(x,y,w,h);
-
-    if (useScale) {
-      draw->PopClip();
       scale->Draw(x,y,w,h);
     }
 
@@ -628,4 +634,41 @@ namespace Lum {
 
     return false;
   }
+
+  Slider* VSlider::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new VSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+
+    return s;
+  }
+
+  Slider* VSlider::Create(Base::Model* model,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new VSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetModel(model);
+
+    return s;
+  }
+
+  Slider* VSlider::Create(Base::Model* model,
+                          bool useScale,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Slider *s;
+
+    s=new VSlider();
+    s->SetFlex(horizontalFlex,verticalFlex);
+    s->SetScale(useScale);
+    s->SetModel(model);
+
+    return s;
+  }
+
 }
