@@ -42,6 +42,7 @@ namespace Lum {
 
     void CalcSize();
     void Layout();
+    void PrepareForBackground(OS::DrawInfo* draw);
     void Draw(int x, int y, size_t w, size_t h);
     void Hide();
 
@@ -54,8 +55,11 @@ namespace Lum {
   };
 
   TabButton::TabButton(size_t index, Object* object, const Model::SizeTRef& model)
-  : image(NULL),model(model),index(index)
+  : image(object),model(model),index(index)
   {
+    assert(image!=NULL);
+    assert(model.Valid());
+
     SetBackground(OS::display->GetFill(OS::Display::tabRiderBackgroundFillIndex));
 
     this->model=model;
@@ -64,8 +68,7 @@ namespace Lum {
     SetCanFocus(true);
     RequestFocus();
 
-    this->image=object;
-    this->image->SetParent(this);
+    image->SetParent(this);
   }
 
   TabButton::~TabButton()
@@ -177,41 +180,33 @@ namespace Lum {
                   height-2*OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder));
     image->Move(x+(width-image->GetOWidth()) / 2,
                 y+(height-image->GetOHeight()) / 2);
+    image->Layout();
 
     Control::Layout();
   }
 
+  void TabButton::PrepareForBackground(OS::DrawInfo* draw)
+  {
+    draw->activated=IsMouseActive();
+    draw->selected=model.Valid() && !model->IsNull() && model->Get()==index;
+    draw->focused=HasFocus();
+  }
+
   void TabButton::Draw(int x, int y, size_t w, size_t h)
   {
-    OS::DrawInfo *draw=GetDrawInfo();
-
-    if (IsMouseActive()) {
-      draw->activated=true;
-    }
-
-    if (!model->IsNull() && model->Get()==index) {
-      draw->selected=true;
-    }
-
-    draw->focused=HasFocus();
-
     Control::Draw(x,y,w,h); /* We must call Draw of our superclass */
 
     if (!OIntersect(x,y,w,h)) {
-      draw->activated=false;
-      draw->selected=false;
-      draw->focused=false;
       return;
     }
 
     /* --- */
 
-    draw->PushClipBegin(x,y,w,h);
-    draw->SubClipRegion(image->GetOX(),image->GetOY(),
-                        image->GetOWidth(),image->GetOHeight());
-    draw->PushClipEnd();
-    DrawBackground(x,y,w,h);
-    draw->PopClip();
+    OS::DrawInfo *draw=GetDrawInfo();
+
+    draw->activated=IsMouseActive();
+    draw->selected=model.Valid() && !model->IsNull() && model->Get()==index;
+    draw->focused=HasFocus();
 
     image->Draw(x,y,w,h);
 
@@ -233,7 +228,7 @@ namespace Lum {
 
   void TabButton::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    if (model==this->model && visible) {
+    if (model==this->model) {
       Redraw();
     }
 
@@ -243,70 +238,72 @@ namespace Lum {
   /* ----------------------- */
 
   Tab::Tab()
-  : mw(0),mh(0),lastDrawn(NULL)
+  : mw(0),mh(0),lastDrawn(NULL),frameGroup(new FrameGroup()),multi(new Multi())
   {
-    SetBackground(OS::display->GetFill(OS::Display::tabBackgroundFillIndex));
-
     SetCanFocus(true);
 
     SetModel(new Model::SizeT(0));
+
+    frameGroup->SetFrame(new OS::EmptyFrame());
+    frameGroup->SetBackground(OS::display->GetFill(OS::Display::tabBackgroundFillIndex));
+    frameGroup->SetFlex(true,true);
+    frameGroup->SetSpace(true);
+
+    multi->SetFlex(true,true);
+    multi->SetModel(GetModel());
+
+    frameGroup->SetObject(multi);
+
     Observe(model);
   }
 
   Tab::~Tab()
   {
-    std::list<Entry>::iterator iter;
-
-    iter=list.begin();
-    while (iter!=list.end()) {
-      delete iter->rider;
-      delete iter->object;
-
-      ++iter;
+    for (std::list<Object*>::iterator iter=list.begin();
+         iter!=list.end();
+         ++iter) {
+      delete *iter;
     }
+
+    delete frameGroup;
   }
 
   bool Tab::VisitChildren(Visitor &visitor, bool onlyVisible)
   {
-    for (std::list<Entry>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
-      if (!visitor.Visit(iter->rider)) {
+    for (std::list<Object*>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
+      if (!visitor.Visit(*iter)) {
         return false;
       }
+    }
 
-      if (onlyVisible) {
-        if (iter->object->IsVisible() && !visitor.Visit(iter->object)) {
-          return false;
-        }
-      }
-      else {
-        if (!visitor.Visit(iter->object)) {
-          return false;
-        }
-      }
+    if (!visitor.Visit(frameGroup)) {
+      return false;
     }
 
     return true;
   }
 
-  void Tab::Add(Object* label, Object* object)
+  Tab* Tab::Add(Object* label, Object* object)
   {
-    assert(label!=NULL && object!=NULL);
+    assert(label!=NULL);
+    assert(object!=NULL);
 
-    Entry entry;
+    Object *rider;
 
-    entry.rider=new TabButton(list.size(),label,model);
-    entry.rider->SetParent(this);
-    entry.rider->SetFlex(true,true);
+    rider=new TabButton(list.size(),label,model);
+    rider->SetParent(this);
+    rider->SetFlex(true,true);
 
-    entry.object=object;
-    entry.object->SetParent(this);
+    list.push_back(rider);
 
-    list.push_back(entry);
+    multi->Append(object);
+
+    return this;
   }
 
-  void Tab::Add(const std::wstring& label, Object* object)
+  Tab* Tab::Add(const std::wstring& label, Object* object)
   {
-    Add(new Text(label),object);
+    return Add(new Text(label),object);
   }
 
   void Tab::ActivatePreviousTab()
@@ -343,13 +340,14 @@ namespace Lum {
 
     Control::SetModel(this->model);
 
+    multi->SetModel(GetModel());
+
     return this->model.Valid();
   }
 
   void Tab::CalcSize()
   {
-    std::list<Entry>::iterator iter;
-    size_t                     ow,oh;
+    size_t ow,oh;
 
     width=0;
     height=0;
@@ -358,26 +356,23 @@ namespace Lum {
     ow=0;
     oh=0;
 
-    iter=list.begin();
-    while (iter!=list.end()) {
-      iter->rider->CalcSize();
-      mw=std::max(mw,(size_t)iter->rider->GetOWidth());
-      mh=std::max(mh,(size_t)iter->rider->GetOHeight());
-
-      iter->object->CalcSize();
-      ow=std::max(ow,(size_t)iter->object->GetOWidth());
-      oh=std::max(oh,(size_t)iter->object->GetOHeight());
-
-      ++iter;
+    for (std::list<Object*>::iterator iter=list.begin();
+         iter!=list.end();
+         ++iter) {
+      (*iter)->CalcSize();
+      mw=std::max(mw,(*iter)->GetOWidth());
+      mh=std::max(mh,(*iter)->GetOHeight());
     }
 
+    frameGroup->SetParent(this);
+    frameGroup->CalcSize();
+    ow=frameGroup->GetOWidth();
+    oh=frameGroup->GetOHeight();
+
     OS::FrameRef tabFrame=OS::display->GetFrame(OS::Display::tabFrameIndex);
 
     mh=std::max(mh,tabFrame->topBorder);
 
-    ow+=2*OS::display->GetSpaceHorizontal(OS::Display::spaceGroupIndent);
-    oh+=2*OS::display->GetSpaceVertical(OS::Display::spaceGroupIndent);
-
     height=mh+oh+tabFrame->minHeight;
     width=std::max(ow+tabFrame->minWidth,
                    list.size()*mw+
@@ -390,21 +385,6 @@ namespace Lum {
     Control::CalcSize();
   }
 
-  Object* Tab::GetObject() const
-  {
-    size_t pos=0;
-
-    for (std::list<Entry>::const_iterator iter=list.begin(); iter!=list.end(); ++iter) {
-      if (pos==model->Get()) {
-        return iter->object;
-      }
-
-      pos++;
-    }
-
-    return NULL;
-  }
-
   void Tab::Layout()
   {
     int     xPos,yPos;
@@ -419,19 +399,18 @@ namespace Lum {
     height=this->height-mh-tabFrame->minHeight;
 
     pos=x+OS::display->GetTheme()->GetFirstTabOffset();
-    for (std::list<Entry>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
+    for (std::list<Object*>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
       // Position rider relative to last rider
-      iter->rider->MoveResize(pos,y,mw,mh);
+      (*iter)->MoveResize(pos,y,mw,mh);
+      (*iter)->Layout();
       pos+=mw;
-
-      // Center object in object area
-      iter->object->Resize(width-2*OS::display->GetSpaceHorizontal(OS::Display::spaceGroupIndent),
-                           height-2*OS::display->GetSpaceVertical(OS::Display::spaceGroupIndent));
-
-      iter->object->Move(xPos+(width-iter->object->GetOWidth()) / 2,
-                         yPos+(height-iter->object->GetOHeight()) / 2);
     }
 
+    frameGroup->Resize(width,height);
+    frameGroup->Move(xPos+(width-frameGroup->GetOWidth()) / 2,
+                     yPos+(height-frameGroup->GetOHeight()) / 2);
+    frameGroup->Layout();
+
     Control::Layout();
   }
 
@@ -448,54 +427,6 @@ namespace Lum {
     OS::DrawInfo *draw=GetDrawInfo();
     OS::FrameRef tabFrame=OS::display->GetFrame(OS::Display::tabFrameIndex);
 
-    lastDrawn=GetObject();
-
-    draw->PushClipBegin(x,y,w,h);
-    // Object
-    draw->SubClipRegion(lastDrawn->GetOX(),lastDrawn->GetOY(),
-                        lastDrawn->GetOWidth(),lastDrawn->GetOHeight());
-    draw->PushClipEnd();
-
-    // Fill background of object with tab control background color
-    DrawBackground(this->x+tabFrame->leftBorder,
-                   this->y+mh+tabFrame->topBorder,
-                   this->width-tabFrame->minWidth,
-                   this->height-mh-tabFrame->minHeight);
-
-    draw->PopClip();
-
-    // Draw all riders
-    for (std::list<Entry>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
-      iter->rider->Draw(x,y,w,h);
-    }
-
-    // Area before the first tab rider
-    if (OS::display->GetTheme()->GetFirstTabOffset()>0) {
-      DrawParentBackground(this->x,
-                           this->y,
-                           OS::display->GetTheme()->GetFirstTabOffset(),
-                           this->mh);
-    }
-
-    // Fill area behind last tab rider with background color of parent control.
-    if (OS::display->GetTheme()->GetFirstTabOffset()+list.size()*mw<this->width) {
-      DrawParentBackground(this->x+OS::display->GetTheme()->GetFirstTabOffset()+list.size()*mw,
-                           this->y,
-                           this->width-list.size()*mw-OS::display->GetTheme()->GetFirstTabOffset(),
-                           this->mh);
-    }
-
-    if (tabFrame->alpha) {
-      draw->PushClipBegin(this->x,this->y+mh,this->width,this->height-mh);
-      draw->SubClipRegion(this->x+tabFrame->leftBorder,
-                          this->y+mh+tabFrame->topBorder,
-                          this->width-tabFrame->minWidth,
-                          this->height-mh-tabFrame->minHeight);
-      draw->PushClipEnd();
-      DrawParentBackground(this->x,this->y+mh,this->width,this->height-mh);
-      draw->PopClip();
-    }
-
     // Draw frame around object area
     tabFrame->SetGap(OS::display->GetTheme()->GetFirstTabOffset()+model->Get()*mw,mw,0);
 
@@ -504,31 +435,14 @@ namespace Lum {
                    this->y+mh,
                    this->width,
                    this->height-mh);
-
-    // Draw currently visible object
-    lastDrawn->Draw(x,y,w,h);
-  }
-
-  void Tab::Hide()
-  {
-    if (visible) {
-      for (std::list<Entry>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
-        iter->rider->Hide();
-      }
-
-      lastDrawn->Hide();
-      lastDrawn=NULL;
-
-      Control::Hide();
-    }
   }
 
   void Tab::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
     if (model==this->model && visible) {
-      size_t                     pos=this->model->Get();
-      std::list<Entry>::iterator iter=list.begin();
-      Dialog                     *dialog=dynamic_cast<Dialog*>(GetWindow()->GetMaster());
+      size_t                       pos=this->model->Get();
+      std::list<Object*>::iterator iter=list.begin();
+      Dialog                       *dialog=dynamic_cast<Dialog*>(GetWindow()->GetMaster());
 
       if (lastDrawn!=NULL) {
         OS::DrawInfo *draw=GetDrawInfo();
@@ -558,10 +472,20 @@ namespace Lum {
           --pos;
         }
 
-        dialog->SetFocus(iter->rider);
+        dialog->SetFocus((*iter));
       }
     }
 
     Control::Resync(model,msg);
   }
+
+  Tab* Tab::Create(bool horizontalFlex, bool verticalFlex)
+  {
+    Tab *object;
+
+    object=new Tab();
+    object->SetFlex(horizontalFlex,verticalFlex);
+
+    return object;
+  }
 }
