@@ -388,15 +388,6 @@ namespace Lum {
     OS::DrawInfo               *draw=GetDrawInfo();
     int                        xPos,yPos,y2;
 
-    if ((draw->focused || DisplayFocus()) && OS::display->GetType()==OS::Display::typeTextual) {
-      draw->PushForeground(OS::Display::fillColor);
-      draw->FillRectangle(this->x,this->y,width,height);
-      draw->PopForeground();
-    }
-    else {
-      DrawBackground(x,y,w,h);
-    }
-
     if ((draw->focused || DisplayFocus()) && OS::display->GetType()==OS::Display::typeTextual) {
       draw->PushForeground(OS::Display::fillTextColor);
     }
@@ -412,7 +403,7 @@ namespace Lum {
       }
     }
 
-    draw->PushClip(this->x,this->y,width,height);
+    draw->PushClip(x,y,w,h);
 
     yPos=this->y;
 
@@ -480,4 +471,28 @@ namespace Lum {
     draw->PopClip();
     draw->PopForeground();
   }
+
+  Text* Lum::Text::Create(const std::wstring& text,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Text *t=new Lum::Text();
+
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetText(text);
+
+    return t;
+  }
+
+  Text* Lum::Text::Create(const std::wstring& text,
+                          Text::Alignment alignment,
+                          bool horizontalFlex, bool verticalFlex)
+  {
+    Text *t=new Lum::Text();
+
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetText(text);
+    t->SetAlignment(alignment);
+
+    return t;
+  }
 }
