@@ -85,18 +85,19 @@ namespace Lum {
         tmp->GetPosAndOffset(pos,offset);
 
         if (pos>=vAdjustment->GetTop() && pos<=vAdjustment->GetTop()+vAdjustment->GetVisible()-1) {
-          DrawItem(GetDrawInfo(),tmp,colHeight*(pos-vAdjustment->GetTop()),offset);
+          Redraw(this->x,this->y+colHeight*(pos-vAdjustment->GetTop()),width,colHeight);
         }
       }
 
       selected=item;
+
       if (selected!=NULL && selected->IsVisible() && visible) {
         size_t pos,offset;
 
         selected->GetPosAndOffset(pos,offset);
 
         if (pos>=vAdjustment->GetTop() && pos<=vAdjustment->GetTop()+vAdjustment->GetVisible()-1) {
-          DrawItem(GetDrawInfo(),selected,colHeight*(pos-vAdjustment->GetTop()),offset);
+          Redraw(this->x,this->y+colHeight*(pos-vAdjustment->GetTop()),width,colHeight);
         }
       }
 
@@ -111,6 +112,25 @@ namespace Lum {
     return selected;
   }
 
+  void Tree::UpdateDimensions()
+  {
+    if (!model.Valid() || model->GetTop()==NULL) {
+      hAdjustment->SetInvalid();
+      vAdjustment->SetInvalid();
+    }
+    else {
+      if (vAdjustment->GetTop()+height/colHeight-1>model->GetVisible()) {
+        vAdjustment->SetDimension(model->GetVisible()-vAdjustment->GetTop()+1,model->GetVisible());
+      }
+      else {
+        vAdjustment->SetDimension(std::min(height/colHeight,model->GetVisible()),model->GetVisible());
+      }
+
+      //TODO: Set hAdjustment
+      hAdjustment->SetDimension(width,width);
+    }
+  }
+
   void Tree::CalcSize()
   {
     colHeight=font->height+2*OS::display->GetSpaceVertical(OS::Display::spaceObjectBorder);
@@ -277,18 +297,19 @@ namespace Lum {
       }
 
       if (item==selected) {
+        OS::FillRef back=OS::display->GetFill(OS::Display::tableBackgroundFillIndex);
+
         draw->selected=true;
-        // TODO
-        DrawBackground(x+colHeight+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject),
-                       this->y+y+(colHeight-font->height)/2,
-                       extent.width,font->height);
+
+        back->Draw(draw,
+                   x+colHeight+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject),
+                   this->y+y+(colHeight-font->height)/2,
+                   extent.width,font->height,
+                   x+colHeight+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject),
+                   this->y+y+(colHeight-font->height)/2,
+                   extent.width,font->height);
         draw->selected=false;
       }
-      else {
-        DrawBackground(x+colHeight+OS::display->GetSpaceHorizontal(OS::Display::spaceIntraObject),
-                       this->y+y+(colHeight-font->height)/2,
-                       extent.width,font->height);
-      }
 
       if (item==selected) {
         if (HasFocus()) {
@@ -313,54 +334,6 @@ namespace Lum {
     }
   }
 
-  void Tree::DrawTree()
-  {
-    OS::DrawInfo *draw=GetDrawInfo();
-
-    DrawBackground(oX,oY,oWidth,oHeight);
-
-    if (model.Valid() && model->GetTop()!=NULL) {
-      size_t            offset;
-      Model::Tree::Item *item;
-
-      if (vAdjustment->GetTop()+height/colHeight-1>model->GetVisible()) {
-        vAdjustment->SetDimension(model->GetVisible()-vAdjustment->GetTop()+1,model->GetVisible());
-      }
-      else {
-        vAdjustment->SetDimension(std::min(height/colHeight,model->GetVisible()),model->GetVisible());
-      }
-
-      draw->PushClip(x,y,width,height);
-      if (model->GetVisibleItem(vAdjustment->GetTop(),item,offset)) {
-        size_t yPos=0;
-
-        while (item!=NULL && yPos<height) {
-          DrawItem(draw,item,yPos,offset);
-
-          if (item->IsExpanded() && item->HasChildren()) {
-            offset++;
-            item=item->GetFirstChild();
-          }
-          else {
-            while (item!=NULL && item->GetNext()==NULL) {
-              item=item->GetParent();
-              --offset;
-            }
-            if (item!=NULL) {
-              item=item->GetNext();
-            }
-          }
-
-          yPos+=colHeight;
-        }
-      }
-      draw->PopClip();
-    }
-    else {
-      vAdjustment->SetDimension(0,0);
-    }
-  }
-
   void Tree::Draw(int x, int y, size_t w, size_t h)
   {
     Scrollable::Draw(x,y,w,h);
@@ -371,27 +344,59 @@ namespace Lum {
 
     /* --- */
 
-    DrawTree();
+    if (!model.Valid() || model->GetTop()==NULL) {
+      return;
+    }
+
+    OS::DrawInfo      *draw=GetDrawInfo();
+    size_t            offset;
+    Model::Tree::Item *item;
+
+    draw->PushClip(x,y,width,height);
+    if (model->GetVisibleItem(vAdjustment->GetTop(),item,offset)) {
+      size_t yPos=0;
+
+      while (item!=NULL && yPos<height) {
+        DrawItem(draw,item,yPos,offset);
+
+        if (item->IsExpanded() && item->HasChildren()) {
+          offset++;
+          item=item->GetFirstChild();
+        }
+        else {
+          while (item!=NULL && item->GetNext()==NULL) {
+            item=item->GetParent();
+            --offset;
+          }
+          if (item!=NULL) {
+            item=item->GetNext();
+          }
+        }
+
+        yPos+=colHeight;
+      }
+    }
+    draw->PopClip();
   }
 
   void Tree::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
     if (model==this->model && visible) {
       if (dynamic_cast<const Model::Tree::ItemCollapsedMsg*>(&msg)!=NULL) {
-        DrawTree();
+        Redraw();
         if (selected!=NULL) {
           SetSelection(selected->GetVisible());
         }
       }
       else if (dynamic_cast<const Model::Tree::ItemExpandedMsg*>(&msg)!=NULL) {
-        DrawTree();
+        Redraw();
       }
       else {
-        DrawTree();
+        Redraw();
       }
     }
     else if (model==vAdjustment->GetTopModel() && visible) {
-      DrawTree();
+      Redraw();
     }
     else {
       Scrollable::Resync(model,msg);
