@@ -25,11 +25,12 @@
 #include <Lum/Base/Util.h>
 
 #include <Lum/OS/Driver.h>
-#include <Lum/OS/KeyboardHandler.h>
 #include <Lum/OS/Theme.h>
 
-#include <Lum/Panel.h>
+#include <Lum/OS/Manager/KeyboardManager.h>
 
+#include <Lum/Panel.h>
+#include <iostream>
 #define MENU_TIMEOUT 2
 
 namespace Lum {
@@ -120,10 +121,6 @@ namespace Lum {
 
   void Window::KeyHandler::SetFocus(Lum::Object* object)
   {
-    if (OS::display->GetKeyboardHandler()!=NULL) {
-      OS::display->GetKeyboardHandler()->OnFocusChange(window,current,object);
-    }
-
     if (current==object) {
       return;
     }
@@ -137,6 +134,10 @@ namespace Lum {
     if (current!=NULL) {
       current->CatchedFocus();
     }
+
+    if (OS::display->GetKeyboardManager()!=NULL) {
+      OS::display->GetKeyboardManager()->OnFocusChange(window,current,object);
+    }
   }
 
   Lum::Object* Window::KeyHandler::GetCurrentFocusObject() const
@@ -199,8 +200,8 @@ namespace Lum {
   void Window::KeyHandler::Activate()
   {
     if (current!=NULL && current->HandlesFocus()) {
-      if (OS::display->GetKeyboardHandler()!=NULL) {
-        OS::display->GetKeyboardHandler()->OnFocusChange(window,current,current);
+      if (OS::display->GetKeyboardManager()!=NULL) {
+        OS::display->GetKeyboardManager()->OnFocusChange(window,current,current);
       }
       current->RecatchedFocus();
     }
@@ -515,6 +516,8 @@ namespace Lum {
 
     if (IsShown()) {
       main->Move(0,0);
+      main->Layout();
+      main->DrawBackground(window->GetDrawInfo(),0,0,window->GetWidth(),window->GetHeight());
       main->Draw(0,0,window->GetWidth(),window->GetHeight());
       keyHandler.SetFocusFirst();
     }
@@ -822,6 +825,8 @@ namespace Lum {
       const OS::Window::RedrawMsg *redrawMsg=dynamic_cast<const OS::Window::RedrawMsg*>(&msg);
 
       main->Move(0,0);
+      main->Layout();
+      main->DrawBackground(window->GetDrawInfo(),redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
       main->Draw(redrawMsg->x,redrawMsg->y,redrawMsg->width,redrawMsg->height);
     }
     else if (model==window->GetPreInitAction() && window->GetPreInitAction()->IsFinished()) {
